/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: submit.cpp 10 2006-02-16 02:31:47Z robert $

----------------------------------------------------------------------------*/
#ifdef WIN32
#if _MSC_VER == 1200
#pragma warning(disable:4786)
#endif
#endif

#include "submit.h"
#include "tunepimp.h"

#define DB printf("%s:%d\n", __FILE__, __LINE__);

//---------------------------------------------------------------------------

int SubmitInfo::getNumItems(void)
{
    return list.size();
}

//---------------------------------------------------------------------------

void SubmitInfo::add(const string &trackId, const string &puidId)
{
    pair<string, string> p;

    if (puidId == string("00000000-0000-0000-0000-000000000000") ||
        puidId == string("40400000-0000-0000-0000-000000000000") ||
        puidId == string("ffffffff-ffff-ffff-ffff-ffffffffffff") ||
        puidId.size() != 36 || trackId.size() != 36)
       return;

    p.first = trackId;
    p.second = puidId;
    list.push_back(p);
}

//---------------------------------------------------------------------------

void SubmitInfo::remove(const string &trackId)
{
    vector<pair<string, string> >::iterator i;

    for(i = list.begin(); i != list.end(); i++)
    {
        if (i->first == trackId)
        {
           list.erase(i);
           return;
        }
    }
}

//---------------------------------------------------------------------------

bool SubmitInfo::find(const string &puidId, string &trackId)
{
    vector<pair<string, string> >::iterator i;

    for(i = list.begin(); i != list.end(); i++)
    {
        if (i->second == puidId)
        {
           trackId = i->first;
           return true;
        }
    }
    return false;
}

//---------------------------------------------------------------------------

void SubmitInfo::getRDF(string &rdf)
{
    vector<pair<string, string> >::iterator i;

    rdf = string(
       "<mq:SubmitPUIDList>\n"
       "<mm:puididList>\n"
       "<rdf:Bag>\n");

    for(i = list.begin(); i != list.end(); i++)
    {
        rdf += string("<rdf:li>\n"
                          "<mq:puidTrackPair>\n"
                          "<mm:puidid>");

        rdf += (*i).second + "</mm:puidid>\n<mm:trackid>";
        rdf += (*i).first + "</mm:trackid>\n</mq:puidTrackPair>\n</rdf:li>\n";
    }

    rdf += string("</rdf:Bag>\n"
                      "</mm:puididList>\n"
                      "<mq:sessionId>@SESSID@</mq:sessionId>\n"
                      "<mq:sessionKey>@SESSKEY@</mq:sessionKey>\n<mq:clientVersion>");
    rdf += appName;
    rdf += string("/");
    rdf += appVersion;
    rdf += string("</mq:clientVersion>\n"
                  "</mq:SubmitPUIDList>\n");
}

//---------------------------------------------------------------------------

LookupStatus SubmitInfo::submit(void)
{
    SubmitPUID   puidSubmit(tunePimp);
    LookupStatus ret;
    string       rdf;

    if (getNumItems() == 0)
    {
        err = "No PUID ids to submit.";
        return eSubmitError;    
    }

    getRDF(rdf);
    puidSubmit.setArgs(rdf, username, password);
    ret = puidSubmit.lookup();
    if (ret != eSubmitOk)
    {
        puidSubmit.getError(err);
        return ret;
    }

    return ret;
}
